#!/usr/bin/env python
# -*- coding: UTF8 -*-

#    pyLeech, downloads files listed in XML and HTML documents.
#    Copyright (C) 2008  Joakim Soderlund
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.


import os
import gtk
import urllib

from SimpleGladeApp import SimpleGladeApp
from SimpleGladeApp import bindtextdomain

from pyleech import pyLeech
from decimal import Decimal
from threading import Thread

__author__ = "Joakim Soderlund"
__version__ = "0.6"
__date__ = "2008-05-02"
__copyright__ = "Copyright (C) 2008  Joakim Soderlund"
__license__ = "GPLv3"

app_name = "pyleech"
app_version = "0.6"

glade_dir = ""
locale_dir = ""

bindtextdomain(app_name, locale_dir)
gtk.gdk.threads_init()

class leech(pyLeech, Thread):
	def __init__(self):
		Thread.__init__(self)
		pyLeech.__init__(self)

	def run(self):
		try:
			urls = plui.entry_url.get_text()
			types = plui.entry_types.get_text()
			target = plui.entry_target.get_text()
			
			self.data = []
			plui.progressbar_file.set_fraction(0.0)
			plui.progressbar_total.set_fraction(0.0)
			plui.progressbar_file.set_text("")
			plui.progressbar_total.set_text("Fetching SGML document...")
			self.getURLs(urls, plui.checkbutton_links.get_active(), plui.checkbutton_images.get_active())
			self.filterURLs(types.split(" "))
			self.delDuplicates()
			plui.progressbar_total.set_text("0 of %d files completed. (00" % len(self.data) + "%)")
			self.dlURLs(target)
		except:
			message = "Error!"
			if not urls:
				message += " Missing URL."
			if not types:
				message += " Missing types."
			if not target:
				message += " Missing target."
			plui.progressbar_total.set_text(message)
				

	def dlURLs(self, folder):
		completedDLs = 0
		for a in self.data:
			filename = a.split("/")[-1]
			fullpath = "%s/%s" % (folder, filename)
			if not os.path.exists(fullpath):
				plui.progressbar_file.set_fraction(0.0)
				plui.progressbar_file.set_text("Downloading %s..." % filename)
				urllib.urlretrieve(a, fullpath, self.progressBar)
				plui.progressbar_file.set_fraction(1.0)
				plui.progressbar_file.set_text("")
			else:
				pass
			completedDLs += 1
			fraction = (Decimal(completedDLs)/Decimal(len(self.data)))
			plui.progressbar_total.set_fraction(fraction)
			plui.progressbar_total.set_text("%d of %d files completed. (%d" % (completedDLs, len(self.data), (fraction * 100)) + "%)")
		plui.progressbar_file.set_fraction(0.0)
		plui.progressbar_total.set_text("Downloaded %s files." % len(self.data))
		if not len(self.data):
			plui.progressbar_total.set_text("Nothing to download.")


class Plui(SimpleGladeApp):
	"""GUI for pyLeech"""
	def __init__(self, path="pyleech.glade", root="plui", domain=app_name, **kwargs):
		path = os.path.join(glade_dir, path)
		SimpleGladeApp.__init__(self, path, root, domain, **kwargs)

	def on_exit_clicked(self, widget, *args):
		gtk.main_quit()

	def on_download_clicked(self, widget, *args):
		self.leech = leech()
		self.leech.progressBar = self.progressBar
		self.leech.start()

	def on_plui_destroy(self, widget, *args):
		gtk.main_quit()

	def progressBar(self, progress, bytesize, target):
		percentage = ((Decimal(progress) * Decimal(bytesize))/Decimal(target))
		if percentage <= Decimal(1):
			self.progressbar_file.set_fraction(percentage)
		else:
			self.progressbar_file.set_fraction(1)

	def on_dirselect_clicked(self, widget, *args):
		folderselect = Folderselect()
		folderselect.folderselect.run()

	def on_typeselect_clicked(self, widget, *args):
		typeselect = Typeselect()
		typeselect.typeselect.run()

	def run(self):
		gtk.gdk.threads_enter()
		gtk.main()
		gtk.gdk.threads_leave()

class Folderselect(SimpleGladeApp):
	def __init__(self, path="pyleech.glade", root="folderselect", domain=app_name, **kwargs):
		path = os.path.join(glade_dir, path)
		SimpleGladeApp.__init__(self, path, root, domain, **kwargs)
	
	def on_folderselect_destroy(self, widget, *args):
		self.folderselect.destroy()
	
	def on_open_clicked(self, widget, *args):
		plui.entry_target.set_text(self.folderselect.get_filename())
		self.folderselect.destroy()

	def on_abort_clicked(self, widget, *args):
		self.folderselect.destroy()

class Typeselect(SimpleGladeApp):
	def __init__(self, path="pyleech.glade", root="typeselect", domain=app_name, **kwargs):
		path = os.path.join(glade_dir, path)
		SimpleGladeApp.__init__(self, path, root, domain, **kwargs)
	
	def on_typeselect_destroy(self, widget, *args):
		self.typeselect.destroy()
	
	def on_apply_clicked(self, widget, *args):
		from sets import Set

		filetypes = plui.entry_types.get_text().split(" ")
		typelists = {
		"images" :    ["jpg", "bmp", "svg", "gif", "png", "tga"],
		"videos" :    ["avi", "mpg", "wmv", "mkv", "ogm", "mp4"],
		"audio" :     ["ogg", "flac", "mp3", "wma", "aac"],
		"archives" :  ["zip", "tar", "gz", "bz2", "7z", "lzma", "rar"],
		"documents" : ["odf", "odt", "pdf", "doc", "docx"]}

		try:
			filetypes.remove("")
		except ValueError:
			pass

		for a in typelists.keys():
			if getattr(self, "cb_%s" % a).get_active():
				filetypes.extend(typelists[a])
		filetypes = list(Set(filetypes))
		filetypes.sort()
		plui.entry_types.set_text(" ".join(filetypes))
		
		self.typeselect.destroy()

	def on_abort_clicked(self, widget, *args):
		self.typeselect.destroy()

if __name__ == "__main__":
	plui = Plui()
	plui.run()

